/*** news.c    - print system news	(C) 1993
****
**** Intruder@link.hacktic.nl
***/

#include <stdio.h>
#include <ctype.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <dirent.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <utime.h>
#include <time.h>
#include <fcntl.h>
#include <pwd.h>

#define VERSION 	"v0.3"
#define PAGER		"more"

char  *myname;
int   cat(char *);
int   more(char *);
int   fcat(int, int);
void  print_usage();

/**
*** MAIN
**/
void main(int argc, char **argv)
{
  DIR	 *dfd;
  struct dirent *dir;
  struct stat st;
  struct passwd *pw;
  time_t nt=0;
  char	 *home, line[1024];
  char	 newsnames[80];
  int	 f_all=0,
	 f_items=0,
	 f_names=0,
	 f_pager=0;
  int	 ch, icount=0, i;
  extern int optind;

  myname = (char *)argv[0];
  for(i=0;i<strlen(argv[0]);i++)
    argv[0][i] = toupper((int)argv[0][i]);

  while ((ch = getopt(argc, argv, "asnp?h")) != EOF)
    switch(ch) {
	   case 'a':            /* show all articles    */
		f_all++;
		break;
	   case 's':
		f_items++;	/* show # of articles	*/
		break;
	   case 'n':
		f_names++;	/* show names of articles */
		break;
	   case 'p':            /* pipe through $PAGER  */
		f_pager++;
		break;
	   case '?':
	   case 'h':
	   default:
		print_usage();
		exit(0);
		break;
		}


  home = getenv("HOME");
  if(!home)
    exit(0);

  sprintf(line, "%s/%s", home, NEWSTIME);
  if(stat(line, &st) == -1)
    {
      if(errno != ENOENT)
	exit(errno);
      if(!f_all && !f_items && !f_names && !argv[optind])
       close(creat(line, 0644));
    }
  else
   nt = st.st_mtime;

 if(!f_all && !f_items && !f_names && !argv[optind])
  if(utime(line, (struct utimbuf *)0) == -1)
   {
     fprintf(stderr, "%s: (%s) %s\n", myname, line, strerror(errno));
   }

  dfd = opendir(NEWSDIR);
  if(!dfd)
   {
     fprintf(stderr, "%s: opendir(%s): %s\n", myname,
		      NEWSDIR, strerror(errno));
     exit(errno);
   }

  while((dir=readdir(dfd)))
   {
     if(!strcmp(dir->d_name, ".") || !strcmp(dir->d_name, ".."))
       continue;
     if(argv[optind])
      if(strncmp(dir->d_name, argv[optind], strlen(argv[optind])))
       continue;
     sprintf(line, "%s/%s", NEWSDIR, dir->d_name);
     if(stat(line, &st) == -1)
      {
	fprintf(stderr, "%s: (%s) %s\n", myname, line, strerror(errno));
	continue;
      }
     if(nt < st.st_mtime || f_all || argv[optind])
      {
	if(!f_names && !f_items)
	 {
	    pw = getpwuid(st.st_uid);
	    printf("\n%s (%s)  %s\n",dir->d_name,
	     pw ? pw->pw_name : "unknown", ctime(&st.st_mtime));
	    if(f_pager)
	     more(line);
	    else
	     cat(line);
	 }
	if(f_names)
	  {
	    if((strlen(dir->d_name) +
		strlen(newsnames) +
		strlen(myname) + 1 + 2) > 80)
	     {
	       printf("%s: %s\n", myname, newsnames);
	       strcpy(newsnames, "");
	     }
	    strcat(newsnames, dir->d_name);
	    strcat(newsnames, " ");
	  }
	icount++;
      }
   }
  if(f_names && icount && newsnames[0])
     printf("%s: %s\n", myname, newsnames);
  if(f_items)
     printf("%s: %d news article%s\n",
      myname, icount, icount==1 ? "" : "s");
  closedir(dfd);
 exit(0);
}

/**
*** print a file to stdout
**/
int cat(char *file)
{
  FILE *fd;

  fd = fopen(file, "r");
  if(!fd)
   {
     fprintf(stderr, "%s: (%s) %s\n", myname, file, strerror(errno));
     return 0;
   }

  fcat(fileno(fd), fileno(stdout));
 fclose(fd);
 return 1;
}

/**
*** pipe a file through $PAGER
**/
int more(char *file)
{
  FILE *fd;
  FILE *pfd;

  fd = fopen(file, "r");
  if(!fd)
    {
       fprintf(stderr, "%s: (%s) %s\n", myname, file, strerror(errno));
       return 0;
     }
   pfd = popen(getenv("PAGER") ? getenv("PAGER") : PAGER, "w");
   if(!pfd)
     {
       fprintf(stderr, "%s: popen() fialed: %s\n", myname, strerror(errno));
       fclose(fd);
       return 0;
     }
  fcat(fileno(fd), fileno(pfd));
  fclose(fd);
  pclose(pfd);
 return 1;
}

int fcat(int fdin, int fdout)
{
  int i;
  char buf[1024];

  while((i=read(fdin, buf, 1024)))
   {
     if(i == -1)
       {
	  fprintf(stderr, "%s: %s\n", myname, strerror(errno));
	  return 0;
       }
     write(fdout, buf, i);
   }
  return 1;
}

void print_usage()
{
  fprintf(stderr, "%s %s  -  Intruder@link.hacktic.nl\n", myname, VERSION);
  fprintf(stderr, "Display system news - Usage: news [-flags] [article]\n\n");
  fprintf(stderr, "        -a     Display all news\n");
  fprintf(stderr, "        -n     Display news article names only\n");
  fprintf(stderr, "        -p     Pipe article through $PAGER\n");
  fprintf(stderr, "        -s     Display number of news articles\n\n");
}
